/****************************************************************************

                                TZlib beta

                   Copyright (c) 1998 by Richard W. Frost

                           Written:   01/06/98
                           Modified:  01/15/98

This file contains the implementation of the TZlib class, a C++ wrapper for the
'zlib' compression and decompression engine.  This class uses the zlib general
purpose compression library version 1.0.4, Jul 24th, 1996 Copyright (C)
1995-1996 by Jean-loup Gailly and Mark Adler.  For more information about zlib
see the zlib.h header file or visit the zlib home page at:

					http://www.cdrom.com/pub/infozip/zlib/

The TZlib class is a C++Builder adaptation and expansion of the freeware C++
ZlibEngine class.  ZlibEngine is a Tiny Software (tm) project written by Mark
Nelson.  Contact markn@tiny.com for more information. Or visit his web site at:

                http://web2.airmail.net/markn/index.html

****************************************************************************/

//---------------------------------------------------------------------------
#include <windows.h>
#include "TZlib.h"

#pragma link "zlib.LIB"
//---------------------------------------------------------------------------
__fastcall TZlib::TZlib()
  : TObject()
// The constructor instantiates the z_stream and initializes a couple of members
// of the z_stream class.  See the zlib documentation for details on what those
// members do           
{
  ZStream = new z_stream();
  ZStream->zalloc = 0;
  ZStream->zfree = 0;
  ZStream->opaque = 0;

  InStream = NULL;
  OutStream = NULL;
  FUserAbort = false;
  FOnZlibProgress = NULL;
  FOnZlibResult = NULL;
}
//---------------------------------------------------------------------------
__fastcall TZlib::~TZlib()
{
  delete ZStream;
}
//---------------------------------------------------------------------------
int TZlib::Compress(TStream *input, TStream *output, int level)
// Compress() takes care of setting up the buffers for zlib and then calls
// deflate() repeatedly until all input and output processing has been done,
// and finally cleans up the zlib structures.
{
    ZlibError = Z_OK;
    ZStream->avail_in = 0;
    ZStream->avail_out = Z_BUFFER_SIZE;
    ZStream->next_out = OutputBuffer;
    UserAbort = false;

    InStream  = input;
    OutStream = output;
    Length = InStream->Size;
    deflateInit(ZStream, level);  // zlib call
    while (!UserAbort && LoadInput())
    {
      ZlibError = deflate(ZStream, Z_NO_FLUSH); // zlib call
      FlushOutput();
      if (ZlibError != Z_OK)
        break;
      Progress(Percent());
    }
    while (!UserAbort)
    {
        ZlibError = deflate(ZStream, Z_FINISH); // zlib call
        if (!FlushOutput())
            break;
        if (ZlibError != Z_OK)
            break;
    }
    Progress(Percent());
    deflateEnd(ZStream); // zlib call
    if (UserAbort)
      Result("User Abort", Z_USER_ABORT);
    else if (ZlibError != Z_OK && ZlibError != Z_STREAM_END)
      Result("Zlib Error", ZlibError);
    else
    {
      Result("Success", Z_OK);
      ZlibError = Z_OK;
    }
    InStream = 0;     // Reset the stream
    OutStream = 0;    // pointers
    if (UserAbort)
      return Z_USER_ABORT;
    else
      return ZlibError;
}
//---------------------------------------------------------------------------
int TZlib::Decompress(TStream *input, TStream *output)
// Decompress has to do most of the same chores as Compress.  The only major
// difference is the absence of the level parameter.  The level isn't needed
// when decompressing data using the deflate algorithm.
{
    ZlibError = Z_OK;
    ZStream->avail_in = 0;
    ZStream->avail_out = Z_BUFFER_SIZE;
    ZStream->next_out = OutputBuffer;
    UserAbort = false;

    InStream  = input;
    OutStream = output;
    Length = InStream->Size;
    inflateInit(ZStream); // zlib call
    while(!UserAbort && LoadInput())
    {
      ZlibError = inflate(ZStream, Z_NO_FLUSH); // zlib call
      FlushOutput();
      if (ZlibError != Z_OK)
        break;
      Progress(Percent());
    }
    while(!UserAbort)
    {
      ZlibError = inflate(ZStream, Z_FINISH); // zlib call
      if (!FlushOutput())
          break;
      if (ZlibError != Z_OK)
          break;
    }
    Progress(Percent());
    inflateEnd(ZStream); // zlib call
    if (UserAbort)
        Result("User Abort", Z_USER_ABORT);
    else if (ZlibError != Z_OK && ZlibError != Z_STREAM_END)
        Result("Zlib Error", ZlibError);
    else
    {
        Result("Success", Z_OK);
        ZlibError = Z_OK;
    }
    InStream = 0;   // Reset the stream
    OutStream = 0;  // pointers
    if (UserAbort)
      return Z_USER_ABORT;
    else
      return ZlibError;
}
//---------------------------------------------------------------------------
int TZlib::Percent()
// This function is called so as to provide the Progress() virtual function
// with a reasonable figure to indicate how much processing has been done.  Note
// that the Length member is initialized when the input file is opened.
{
  if (Length == 0)
    return 100;
  else if (Length > 10000000L)
    return (ZStream->total_in /(Length / 100));
  else
    return (ZStream->total_in * 100 / Length);
}
//---------------------------------------------------------------------------
int TZlib::LoadInput()
// Every time zlib consumes all of the data in the input buffer, this function
// gets called to reload. The avail_in member is part of z_stream, and is used
// to keep track of how much input is available.  The windows message loop is
// churned so that the process can be aborted by a button press or other
// Windows event.
{
  MSG message;
  while (PeekMessage(&message, NULL, 0, 0, PM_REMOVE))
  {
    TranslateMessage(&message);
    DispatchMessage(&message);
  }
  if (ZStream->avail_in == 0)
  {
    ZStream->next_in = InputBuffer;
    ZStream->avail_in = InStream->Read(InputBuffer, Z_BUFFER_SIZE);
  }
  return ZStream->avail_in;
}
//---------------------------------------------------------------------------
int TZlib::FlushOutput()
// Every time zlib fills the output buffer with data, FlushOutput gets called.
// Its job is to write that data out to the output stream, then update the
// z_stream member avail_out to indicate that more space is now available.  The
// Windows message loop is churned to ensure that the process can be aborted by
// a button press or other Windows event.
{
  MSG message;
  while (PeekMessage(&message, NULL, 0, 0, PM_REMOVE))
  {
    TranslateMessage(&message);
    DispatchMessage(&message);
  }
  unsigned int count = Z_BUFFER_SIZE - ZStream->avail_out;
  if (count)
  {
    OutStream->WriteBuffer(OutputBuffer, count);
    ZStream->next_out = OutputBuffer;
    ZStream->avail_out = Z_BUFFER_SIZE;
  }
  return count;
}
//---------------------------------------------------------------------------
inline void TZlib::Progress(int percent)
// Triggers the OnZlibProgress event
{
  if (FOnZlibProgress)
    OnZlibProgress(this, percent);
}
//---------------------------------------------------------------------------
inline void TZlib::Result(AnsiString message, int zresult)
// Triggers the OnZlibResult event
{
  if (FOnZlibResult)
    OnZlibResult(this, message, zresult);
}
//---------------------------------------------------------------------------
void __fastcall TZlib::SetUserAbort(bool value)
// The setter for the UserAbort property
{
  if (FUserAbort != value)
  {
    FUserAbort = value;
    if (FUserAbort)
      ZlibError = Z_USER_ABORT;
  }
}
//---------------------------------------------------------------------------


